/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 *   Pierre Phaneuf <pp@ludusdesign.com>
 */
#include "nsIComponentManager.h" 
#include "nsMimeObjectClassAccess.h"
#include "nsCOMPtr.h"
#include "nsMsgMimeCID.h"

static NS_DEFINE_CID(kMimeObjectClassAccessCID, NS_MIME_OBJECT_CLASS_ACCESS_CID); 

/*
 * These calls are necessary to expose the object class heirarchy 
 * to externally developed content type handlers.
 */
extern "C" void *
COM_GetmimeInlineTextClass(void)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  void                                *ptr = NULL;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->GetmimeInlineTextClass(&ptr);

  return ptr;
}

extern "C" void *
COM_GetmimeLeafClass(void)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  void                                *ptr = NULL;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->GetmimeLeafClass(&ptr);

  return ptr;
}

extern "C" void *
COM_GetmimeObjectClass(void)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  void                                *ptr = NULL;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->GetmimeObjectClass(&ptr);

  return ptr;
}

extern "C" void *
COM_GetmimeContainerClass(void)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  void                                *ptr = NULL;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->GetmimeContainerClass(&ptr);

  return ptr;
}

extern "C" void *
COM_GetmimeMultipartClass(void)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  void                                *ptr = NULL;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->GetmimeMultipartClass(&ptr);

  return ptr;
}

extern "C" void *
COM_GetmimeMultipartSignedClass(void)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  void                                *ptr = NULL;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->GetmimeMultipartSignedClass(&ptr);

  return ptr;
}

extern "C" int  
COM_MimeObject_write(void *mimeObject, char *data, PRInt32 length, 
                     PRBool user_visible_p)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  PRInt32                             rc=-1;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
  { 
    if (NS_SUCCEEDED(objAccess->MimeObjectWrite(mimeObject, data, length, user_visible_p)))
      rc = length;
    else
      rc = -1;
  } 

  return rc;
}

extern "C" void *
COM_MimePartBufferCreate(void)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  void                                *ptr = NULL;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->MimePartBufferCreate(&ptr);

  return ptr;
}

extern "C" void 
COM_MimePartBufferClose(void *data)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->MimePartBufferClose(data);

  return;
}

extern "C" int 
COM_MimePartBufferWrite(void *data,
              const char *buf, PRInt32 size)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  PRInt32                             rc=-1;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    if (NS_SUCCEEDED(objAccess->MimePartBufferWrite(data, buf, size)))
    rc = size;

  return rc;
}

extern "C" int 
COM_MimePartBufferRead (void *data,
              int (*read_fn) (char *buf, PRInt32 size, 
              void *closure), void *closure)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  PRInt32                             rc=-1;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    if (NS_SUCCEEDED(objAccess->MimePartBufferRead(data, read_fn, closure)))
    rc = 0;

  return rc;
}

extern "C" void *
COM_MimeHeaders_new (void)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  void                                *ptr = NULL;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->MimeHeaders_new(&ptr);

  return ptr;
}

extern "C" void
COM_MimeHeaders_free (void *hdrs)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->MimeHeaders_free(hdrs);

  return;
}

extern "C" void
COM_MimeHeaders_parse_line (const char *buffer, PRInt32 size, void *hdrs)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->MimeHeaders_parse_line(buffer, size, hdrs);

  return;
}

extern "C" char *
COM_MimeHeaders_get (void *hdrs, const char *header_name,
            PRBool strip_p, PRBool all_p)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  char                                *content_type = NULL;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->MimeHeaders_get(hdrs, header_name, strip_p, all_p, &content_type);

  return content_type;
}

extern "C" void 
COM_mime_free (void *object)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->mime_free(object);

  return;
}

extern "C" void * 
COM_mime_create (const char *content_type, void *hdrs,
           void *opts)
{
  nsCOMPtr<nsIMimeObjectClassAccess>  objAccess;
  void                                *ptr = NULL;

  nsresult res = nsComponentManager::CreateInstance(kMimeObjectClassAccessCID, 
                                                    NULL, NS_GET_IID(nsIMimeObjectClassAccess), 
                                                    (void **) getter_AddRefs(objAccess)); 
  if (NS_SUCCEEDED(res) && objAccess)
    objAccess->mime_create(content_type, hdrs, opts, &ptr);

  return ptr;
}

